package PayProp::API::Public::Client::Exception::Storage;
use parent qw/ PayProp::API::Public::Client::Exception::Base /;

use strict;
use warnings;

=head1 NAME

	PayProp::API::Public::Client::Exception::Storage - Custom exception.

=head1 SYNOPSIS

	PayProp::API::Public::Client::Exception::Storage->throw('Oh no!');

=head1 DESCRIPTION

Throw C<PayProp::API::Public::Client::Exception::Storage> exception.

=cut

1;
